@@ -3,7 +3,7 @@ our $AUTHORITY = 'cpan:GENE';
 
 # ABSTRACT: Control-change based RtController filters
 
-our $VERSION = '0.0908';
+our $VERSION = '0.0909';
 
 use v5.36;
 
@@ -110,6 +110,13 @@ has running => (
 );
 
 
+has halt => (
+    is      => 'rw',
+    isa     => Bool,
+    default => 0,
+);
+
+
 sub add_filters ($filters, $controllers) {
     for my $params (@$filters) {
         my $port = delete $params->{port};
@@ -161,9 +168,16 @@ sub breathe ($self, $device, $dt, $event) {
         IO::Async::Timer::Periodic->new(
             interval  => $self->time_step,
             on_tick => sub {
-                $it->iterate;
-                my $cc = [ 'control_change', $self->channel, $self->control, $it->i ];
-                $self->rtc->send_it($cc);
+                my ($c) = @_;
+                if ($self->halt) {
+                    $c->stop;
+                    $self->running(0);
+                }
+                else {
+                    $it->iterate;
+                    my $cc = [ 'control_change', $self->channel, $self->control, $it->i ];
+                    $self->rtc->send_it($cc);
+                }
             },
         )->start
     );
@@ -188,9 +202,16 @@ sub scatter ($self, $device, $dt, $event) {
         IO::Async::Timer::Periodic->new(
             interval  => $self->time_step,
             on_tick => sub {
-                my $cc = [ 'control_change', $self->channel, $self->control, $value ];
-                $self->rtc->send_it($cc);
-                $value = $values[ int rand @values ];
+                my ($c) = @_;
+                if ($self->halt) {
+                    $c->stop;
+                    $self->running(0);
+                }
+                else {
+                    my $cc = [ 'control_change', $self->channel, $self->control, $value ];
+                    $self->rtc->send_it($cc);
+                    $value = $values[ int rand @values ];
+                }
             },
         )->start
     );
@@ -220,25 +241,32 @@ sub stair_step ($self, $device, $dt, $event) {
         IO::Async::Timer::Periodic->new(
             interval  => $self->time_step,
             on_tick => sub {
-                my $cc = [ 'control_change', $self->channel, $self->control, $value ];
-                $self->rtc->send_it($cc);
-
-                # compute the stair-stepping
-                if ($direction) {
-                    $it->step($self->step_up);
+                my ($c) = @_;
+                if ($self->halt) {
+                    $c->stop;
+                    $self->running(0);
                 }
                 else {
-                    $it->step(- $self->step_down);
+                    my $cc = [ 'control_change', $self->channel, $self->control, $value ];
+                    $self->rtc->send_it($cc);
+
+                    # compute the stair-stepping
+                    if ($direction) {
+                        $it->step($self->step_up);
+                    }
+                    else {
+                        $it->step(- $self->step_down);
+                    }
+
+                    # toggle the stair-step direction
+                    $direction = !$direction;
+
+                    # iterate the breathing
+                    $it->iterate;
+                    $value = $it->i;
+                    $value = $self->range_top    if $value >= $self->range_top;
+                    $value = $self->range_bottom if $value <= $self->range_bottom;
                 }
-
-                # toggle the stair-step direction
-                $direction = !$direction;
-
-                # iterate the breathing
-                $it->iterate;
-                $value = $it->i;
-                $value = $self->range_top    if $value >= $self->range_top;
-                $value = $self->range_bottom if $value <= $self->range_bottom;
             },
         )->start
     );
@@ -261,19 +289,24 @@ sub ramp_up ($self, $device, $dt, $event) {
         IO::Async::Timer::Countdown->new(
             delay     => $self->time_step,
             on_expire => sub {
-                my $c = shift;
-
-                my $cc = [ 'control_change', $self->channel, $self->control, $value ];
-                $self->rtc->send_it($cc);
-
-                $value += $self->range_step;
-
-                if ($value > $self->range_top) {
+                my ($c) = @_;
+                if ($self->halt) {
                     $c->stop;
                     $self->running(0);
                 }
                 else {
-                    $c->start;
+                    my $cc = [ 'control_change', $self->channel, $self->control, $value ];
+                    $self->rtc->send_it($cc);
+
+                    $value += $self->range_step;
+
+                    if ($value > $self->range_top) {
+                        $c->stop;
+                        $self->running(0);
+                    }
+                    else {
+                        $c->start;
+                    }
                 }
             },
         )->start
@@ -298,19 +331,24 @@ sub ramp_down ($self, $device, $dt, $event) {
         IO::Async::Timer::Countdown->new(
             delay     => $self->time_step,
             on_expire => sub {
-                my $c = shift;
-
-                my $cc = [ 'control_change', $self->channel, $self->control, $value ];
-                $self->rtc->send_it($cc);
-
-                $value -= $self->range_step;
-
-                if ($value < $self->range_bottom) {
+                my ($c) = @_;
+                if ($self->halt) {
                     $c->stop;
                     $self->running(0);
                 }
                 else {
-                    $c->start;
+                    my $cc = [ 'control_change', $self->channel, $self->control, $value ];
+                    $self->rtc->send_it($cc);
+
+                    $value -= $self->range_step;
+
+                    if ($value < $self->range_bottom) {
+                        $c->stop;
+                        $self->running(0);
+                    }
+                    else {
+                        $c->start;
+                    }
                 }
             },
         )->start
@@ -333,7 +371,7 @@ MIDI::RtController::Filter::CC - Control-change based RtController filters
 
 =head1 VERSION
 
-version 0.0908
+version 0.0909
 
 =head1 SYNOPSIS
 
@@ -509,6 +547,15 @@ Are we running a filter?
 
 Default: C<0>
 
+=head2 halt
+
+  $halt = $filter->halt;
+  $filter->halt($boolean);
+
+Return or set B<halt>. This can be used to terminate B<running> filters.
+
+Default: C<0>
+
 =head1 METHODS
 
 =head2 new
